﻿Type=Class
Version=3
@EndOfDesignText@
'Class module
Sub Class_Globals

	Private mTarget As Object
    Private mEventName As String
	Private mAct As Activity
	Private mBase As Panel
	Private DesignerCVCalled As Boolean
	Private Initialized As Boolean
	Private CBox As CheckBox

	
End Sub

'Initializes the object.
'Required for both designer and code setup. 
'Provides callback {EventName}_CheckedChange(Checked As Boolean)
Public Sub Initialize (TargetModule As Object, EventName As String)
	mTarget = TargetModule
	mEventName = EventName
	CBox.Initialize("CB")

End Sub
'Used by the designer added customview DO NOT CALL THIS SUB DIRECTLY
Public Sub DesignerCreateView(Base As Panel, Lbl As Label, Props As Map)
	'Check this is not called from setup
	If Not(Props.GetDefault("SLfromsetup",False)) Then DesignerCVCalled = True
	
	'Assign vars to globals
	mBase = Base
	'This is passed from either the designer or setup
	mAct = Props.get("activity")
	
	'Add the checkbox to the base panel
	mBase.AddView(CBox,0,0,mBase.Width,mBase.Height)
	
	'Redraw the check box so it will be centered in the base panel
	SetCBDrawable(CBox,Colors.LightGray,1,Colors.Cyan,Chr(8730),Colors.DarkGray,mBase.Width - 10,10)
	
	mBase.Tag = mEventName
	'Finally
	Initialized = True
End Sub
'Use initialize and setup subs to add a custom checkbox from code
'Pass an initialized panel if you want to set up the customview on a panel, or null to add it to the activity
Sub Setup(Act As Activity,Pnl As Panel,Left As Int,Top As Int,Width As Int,Height As Int)
	'Check if DesignerCreateView has been called
	If DesignerCVCalled Then
		LogColor("You should not call setup if you have defined this view in the designer",Colors.Red)
		Return
	End If
	
	mAct = Act
	
	'Create our own base panel
	Dim Base As Panel
	Base.Initialize("")
	
	'If Null was passed, a Panel wii be created by casting, but it won't be initialized
	If Pnl.IsInitialized Then
		Pnl.AddView(Base,Left,Top,Width,Height)
	Else
		Act.AddView(Base,Left,Top,Width,Height)
	End If
		
	'Set up vars to pass to DesignerCreateView so we don't have to maintain two identical subs
	Dim M As Map
	M.Initialize
	M.Put("activity",mAct)
	'As we are passing a map, we can use it to pass an additional flag for our own use 
	'with an ID unlikely To be used by B4a In the future
	M.Put("SLfromsetup",True)
	
	'We need a label to pass to DesignerCreateView
	Dim L As Label
	L.Initialize("")
	'Default text size
	'Default size for custom view text in designer is 14
	'Default For a Spinner Is 16, I didn't want to enforce this as you wouldn't then be able to change the size in the designer
	'and a default SLSpinner added in designer would look different to one added in code.
	L.TextSize = 14						
	L.Typeface = Typeface.DEFAULT
	L.Gravity = Gravity.CENTER_VERTICAL
	'Leave TextColor as default. Color is write only so we can't copy it, don't bother setting it
	
	DesignerCreateView(Base,L,M)
	
End Sub
#Region Working Subs
Sub CB_CheckedChange(Checked As Boolean)
	If SubExists(mTarget,mEventName & "_CheckedChange") Then
		CallSub2(mTarget,mEventName & "_CheckedChange",Checked)
	End If
End Sub
#End Region Working subs
#Region Standard Panel Methods
'These act on the Base panel as passed by the designer or the setup from code
'

'Sets the background of the Button to be a ColorDrawable with the given color.
Sub setColor(Color As Int)
	mBase.Color = Color
End Sub
'Changes the Z order of this view and brings it to the front.
Sub BringToFront
	mBase.BringToFront
End Sub
'Gets or sets the height of the displayed View
Sub setHeight(Height As Int)
	mBase.Height = Height
End Sub
Sub getHeight As Int
	Return mBase.Height
End Sub
' Invalidates the view, forcing it to redraw itself.
Sub Invalidate
	mBase.Invalidate
End Sub
'Returns True if the View has been initialized.  For a manually added Spinner, setup must have also been called
Sub IsInitialized As Boolean
	Return Initialized
End Sub
'Gets or sets the view's left position.
Sub setLeft(Left As Int)
	mBase.Left = Left
End Sub
Sub getLeft As Int
	Return mBase.Left
End Sub
'Tries To set the focus To this View.
'Returns True If the focus was set.
Sub RequestFocus As Boolean

End Sub
'Changes the Z order of this view and sends it to the back.
Sub SendToBack
	mBase.SendToBack
End Sub
'Get or Set the background for the Base Panel. Can be one of: BitmapDrawable, ColorDrawable, GradientDrawable or StateListDrawable
'Returns a drawable of the type that was initially or subsequently set
Sub setBackground(BG As Object)
	If Not(BG Is BitmapDrawable OR BG Is ColorDrawable OR BG Is GradientDrawable OR BG Is StateListDrawable) Then
		LogColor("SLSpinner: setBackground failed : Not a drawable Object",Colors.Red)
		Return
	End If
	mBase.Background = BG
End Sub
Sub getBackground As Object
	Return mBase.Background
End Sub
'Changes the view position and size.
Sub SetLayout(Left As Int,Top As Int,Width As Int,Height As Int)
	mBase.SetLayout(Left,Top,Width,Height)
End Sub
'Gets or sets the Tag value. This is a place holder which can used to store additional data.
Sub setTag(Tag As Object)
	mBase.Tag = Tag
End Sub
Sub getTag As Object
	Return mBase.Tag
End Sub
'Gets or sets the views top position.
Sub getTop As Int
	Return mBase.Top
End Sub
Sub setTop(Top As Int)
	mBase.Top = Top
End Sub
'Gets or sets the views Visibility
Sub setVisible(Visible As Boolean)
	mBase.Visible = Visible
End Sub
Sub getVisible As Boolean
	Return mBase.Visible
End Sub
'Gets or sets the view's width. If you have set listwidth, setting this will only effect the Button width
Sub setWidth(Width As Int)
	mBase.Width = Width
End Sub
Sub getWidth As Int
	Return mBase.Width
End Sub
#End Region Standard Panel Methods
#Region CustomView Specific Methods
#End Region CustomView Specific Methods
#Region Utils
'Change the size and color of a Checkbox graphic. Set the tick character and color, as well as the box size and color
'and padding (distance from the box to the edge of the graphic) and a disabled fill color
'Pass "Fill" as the TickChar to fill the box with TickColor when selected.
Sub SetCBDrawable(CB As CheckBox,BoxColor As Int,BoxLineWidth As Int,TickColor As Int,TickChar As String,DisabledColor As Int,Size As Int,Padding As Int)
    Dim SLD As StateListDrawable
    SLD.Initialize

    Dim BMEnabled,BMChecked,BMDisabled As Bitmap
    BMEnabled.InitializeMutable(Size,Size)
    BMChecked.InitializeMutable(Size,Size)
    BMDisabled.InitializeMutable(Size,Size)
    'Draw Enabled State
    Dim CNV As Canvas
    CNV.Initialize2(BMEnabled)
    Dim Rect1 As Rect
    Rect1.Initialize(Padding ,Padding ,Size - Padding ,Size - Padding)
    CNV.DrawRect(Rect1,BoxColor,False,BoxLineWidth)
    Dim Enabled,Checked,Disabled As BitmapDrawable
    Enabled.Initialize(BMEnabled)
    'Draw Selected state
    Dim CNV1 As Canvas
    CNV1.Initialize2(BMChecked)
    If TickChar = "Fill" Then
        CNV1.DrawRect(Rect1,TickColor,True,BoxLineWidth)
        CNV1.DrawRect(Rect1,BoxColor,False,BoxLineWidth)
    Else
        CNV1.DrawRect(Rect1,BoxColor,False,BoxLineWidth)
        'Start small and find the largest font that allows the tick to fit in the box
        Dim FontSize As Int = 6
        Do While CNV.MeasureStringHeight(TickChar,Typeface.DEFAULT,FontSize) < Size - (BoxLineWidth * 2) - (Padding * 2)
            FontSize = FontSize + 1
        Loop
        FontSize = FontSize - 1
        'Draw the TickChar centered in the box
        CNV1.DrawText(TickChar,Size/2,(Size + CNV.MeasureStringHeight(TickChar,Typeface.DEFAULT,FontSize))/2,Typeface.DEFAULT,FontSize,TickColor,"CENTER")
    End If
    Checked.Initialize(BMChecked)
    'Draw disabled State
    Dim CNV2 As Canvas
    CNV2.Initialize2(BMDisabled)
    CNV2.DrawRect(Rect1,DisabledColor,True,BoxLineWidth)
    CNV2.DrawRect(Rect1,BoxColor,False,BoxLineWidth)
    Disabled.Initialize(BMDisabled)

    'Add to the StateList Drawable
    SLD.AddState(SLD.State_Disabled,Disabled)
    SLD.AddState(SLD.State_Checked,Checked)
    SLD.AddState(SLD.State_Enabled,Enabled)
    SLD.AddCatchAllState(Enabled)
    'Add SLD to the Checkbox
    Dim JO As JavaObject = CB
    JO.RunMethod("setButtonDrawable",Array As Object(SLD))
End Sub
#End Region Utils